#include <windows.h>
#include <tchar.h>
#include <math.h>

#if defined(_WIN32_WCE)
    #include <aygshell.h>
#endif

#include "pockethal.h"

using namespace PHAL;


#include "image.h"


TCHAR *lpszClassName = _T("PHal");
TCHAR *lpszCaption = _T("RotoZoom");


WNDCLASS wc;
MSG msg;


Display *display;


bool quit = false;


#define ANGLESTEP 512
int costable[ANGLESTEP];
int sintable[ANGLESTEP];

unsigned short int palette[256];



#define PI 3.14159265f
#define RGB16(r, g, b) ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | ((b & 0xF8 ) >> 3)



void init() {

    float pistep = 2*PI/ANGLESTEP;
    float pi = 0;
    int i;

    for (i = 0; i < ANGLESTEP; i++) {
        costable[i] = (int)(cos(pi)*256*1.35);
        sintable[i] = (int)(sin(pi)*256*1.35);

        pi += pistep;
    }

    for (i = 0; i < 256; i++) {
        palette[i] = RGB16(i/2, i/2+64, i/2+128);
    }


}




void drawFrame()
{
    static unsigned int idxpalette = 128;
    static unsigned int time = GetTickCount();
    unsigned int time2 = GetTickCount();
    if (time2 - time < 30)
        return;
    
    int speed = ((time2 - time)<<8) / 30;

    time = time2;
   
    static int angle = 0;
    static int idx = 0;
 

    idx += speed;
    if (idx >= ANGLESTEP<<8)
        idx -= ANGLESTEP<<8;
    int angle2 = sintable[idx>>8];

    idxpalette += (angle2 - angle);
    idxpalette %= 512;
    if (idxpalette < 0)
        idxpalette += 511;

    angle = angle2;



    int cosA, sinA;
    if (angle > 0) {
        cosA = costable[angle];
        sinA = sintable[angle>>1];
    }
    else {
        cosA = costable[-angle];
        sinA = -sintable[-angle];
    }

    
    unsigned char *src = (unsigned char *)image;
    unsigned short int *pal = (unsigned short int *)palette;
    unsigned short int *dst = (unsigned short int *)display->GetBackBuffer()->GetPixels();

    unsigned int x, y;
    int caos = 0;
    for (y = 0; y < display->GetBackBuffer()->GetHeight(); y++) {

        int yO = y -(display->GetBackBuffer()->GetHeight()>>1)+15;

        for (x = 0; x < display->GetBackBuffer()->GetWidth(); x++) {
            
            int xO = x -(display->GetBackBuffer()->GetWidth()>>1)+15;
            int xT = (cosA*xO - sinA*yO)<<8;
            int yT = sinA*xO + cosA*yO;

            xT += caos;
            caos += 0x00CF;
          
            int px = (xT >> 16) & 0x7F;
            if (px < 0)
                px += 128;

            
            int py = (yT >> 8) & 0x7F;
            if (py < 0)
                py += 128;

            

            unsigned int i = src[(py<<7)+px];
            i += idxpalette;
            if (i >= 512)
                i -= 512;

            unsigned short int c;
            if (i <= 255)
                c = pal[i];
            else
                c = pal[255-(i-256)];

         
            *dst++ = c;
        }
    }


    display->Swap();
}






LRESULT CALLBACK WindowProc(HWND hWnd, unsigned uMsg, WPARAM wParam, LPARAM lParam) {


    switch (uMsg) {

        case WM_LBUTTONDOWN: 
        case WM_KEYDOWN:
            quit = true;
          return 0;
        

        case WM_CLOSE:
            PostQuitMessage(0);
          return 0;
    }

    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, LPTSTR pCmdLine, int nCmdShow)
{
  
  wc.style          = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc    = WindowProc;
  wc.cbClsExtra     = 0;
  wc.cbWndExtra     = sizeof(DWORD);
  wc.hInstance      = hInstance;
  wc.hIcon          = NULL;
  wc.hCursor        = NULL;
  wc.hbrBackground  = 0;
  wc.lpszMenuName   = NULL;
  wc.lpszClassName  = lpszClassName;

  if (RegisterClass(&wc))
  {
    HWND hWnd = CreateWindowEx(WS_EX_TOPMOST,
                            lpszClassName,
                            lpszCaption,
                            WS_VISIBLE,
                            CW_USEDEFAULT,    // x
                            CW_USEDEFAULT,    // y 
                            CW_USEDEFAULT,
                            CW_USEDEFAULT,
                            NULL,
                            NULL,
                            hInstance,
                            0);


    if (hWnd)
    {

    #if defined(_WIN32_WCE)
      // hide all adornments if we are running Windows CE
      SHFullScreen(hWnd, SHFS_HIDESTARTICON | SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON);
      // hide the task bar
      HWND hWndTaskbar = ::FindWindow(TEXT("HHTaskbar"), NULL);
      ShowWindow(hWndTaskbar, SW_HIDE);
    #endif

      display = Display::Create(hWnd);
      if (display == 0)
        return 0;


      ShowWindow(hWnd, nCmdShow);
      UpdateWindow(hWnd);
      ShowCursor(false);


      init();


      while (msg.message != WM_QUIT)
      {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
        }
        else
        {
            if (quit)
                break;

            drawFrame();
        }
      }


      delete display;

    #if defined(_WIN32_WCE)
      // show the task bar
      hWndTaskbar = ::FindWindow(TEXT("HHTaskbar"), NULL);
      ShowWindow(hWndTaskbar, SW_SHOW);
      // restore all adornments if we are running Windows CE
      SHFullScreen(hWnd, SHFS_SHOWSTARTICON | SHFS_SHOWTASKBAR | SHFS_SHOWSIPBUTTON);
    #endif

      if (DestroyWindow(hWnd))
        UnregisterClass(lpszClassName, hInstance);

    }

  }
  
            
  return msg.wParam;
}
